@@ -7,12 +7,25 @@
 #  see <http://www.gnu.org/licenses/>.
 
 from . import decomp, geometry, migration
-from .geometry import Disk, Cube, Rectangle
+from .geometry import (
+    Ball,
+    Disk,
+    Cube,
+    Cylinder,
+    Rectangle,
+    Union,
+    Intersection,
+    Difference,
+    Repeat,
+    Torus,
+    Prism,
+)
 from .generation import generate_mesh, sliver_removal
 from .sizing import (
     get_sizing_function_from_segy,
     write_velocity_model,
     plot_sizing_function,
+    SizeFunction,
 )
 from .__about__ import __version__
 
@@ -22,7 +35,14 @@ __all__ = [
     "geometry",
     "Rectangle",
     "Cube",
+    "Cylinder",
     "Disk",
+    "Union",
+    "Torus",
+    "Prism",
+    "Ball",
+    "Intersection",
+    "Difference",
     "get_sizing_function_from_segy",
     "write_velocity_model",
     "plot_sizing_function",
@@ -30,4 +50,6 @@ __all__ = [
     "sliver_removal",
     "decomp",
     "migration",
+    "SizeFunction",
+    "Repeat",
 ]
