@@ -114,6 +114,8 @@ def sliver_removal(points, domain, edge_length, comm=None, **kwargs):  # noqa: C
         "delta_t": 0.30,
         "geps_mult": 0.1,
         "subdomains": [],
+        "gamma": 1.0,
+        "preserve": False,
     }
 
     sliver_opts.update(kwargs)
@@ -136,7 +138,7 @@ def sliver_removal(points, domain, edge_length, comm=None, **kwargs):  # noqa: C
 
     fd, bbox0, _ = _unpack_domain(domain, sliver_opts)
 
-    fh, bbox1, hmin = _unpack_sizing(edge_length)
+    fh, bbox1, hmin, _ = _unpack_sizing(edge_length)
 
     # take minmax of boxes for the case of domain padding
     if bbox1 is None:
@@ -170,6 +172,7 @@ def sliver_removal(points, domain, edge_length, comm=None, **kwargs):  # noqa: C
     )
 
     geps = sliver_opts["geps_mult"] * h0
+
     deps = np.sqrt(np.finfo(np.double).eps) * h0
     min_dh_bound = sliver_opts["min_dh_angle_bound"] * math.pi / 180
     max_dh_bound = sliver_opts["max_dh_angle_bound"] * math.pi / 180
@@ -197,10 +200,12 @@ def sliver_removal(points, domain, edge_length, comm=None, **kwargs):  # noqa: C
 
     count = 0
     pold = None
-    push = 0.10
+    step = 0.10
+    gamma = sliver_opts["gamma"]
+    num_old_bad = np.inf
 
     dt = DT()
-    dt.insert(p.flatten().tolist())
+    dt.insert(p.ravel().tolist())
     while True:
 
         start = time.time()
@@ -209,7 +214,7 @@ def sliver_removal(points, domain, edge_length, comm=None, **kwargs):  # noqa: C
         # Using CGAL's incremental Delaunay triangulation capabilities.
         if count > 0:
             to_move = np.where(_dist(p, pold) > 0)[0]
-            dt.move(to_move.flatten().tolist(), p[to_move].flatten().tolist())
+            dt.move(to_move.ravel().tolist(), p[to_move].ravel().tolist())
 
         # Get the current topology of the triangulation
         p, t = _get_topology(dt)
@@ -224,7 +229,7 @@ def sliver_removal(points, domain, edge_length, comm=None, **kwargs):  # noqa: C
         # Number of iterations reached, stop.
         if count == (max_iter - 1):
             print_msg1(
-                "FAILURE: Termination...maximum number of iterations reached.",
+                "FAILURE: Termination...maximum number of iterations reached. Try increasing max_iter when generating the mesh",
             )
             p, t = _termination(p, t, sliver_opts, comm, sliver=True)
             break
@@ -248,7 +253,6 @@ def sliver_removal(points, domain, edge_length, comm=None, **kwargs):  # noqa: C
                 + " iterations...no slivers detected!",
             )
             p, t, _ = geometry.fix_mesh(p, t, dim=dim, delete_unused=True)
-            p = _improve_level_set_newton(p, t, fd, deps, deps * 1000)
             return p, t
 
         p0, p1, p2, p3 = (
@@ -266,11 +270,30 @@ def sliver_removal(points, domain, edge_length, comm=None, **kwargs):  # noqa: C
         perturb_norm = np.sum(np.abs(perturb) ** 2, axis=-1) ** (1.0 / 2)
         perturb /= perturb_norm[:, None]
 
-        # perturb push % of minimum mesh size
-        p[move] += push * h0 * perturb
+        num_bad = len(ele_nums)
+
+        if num_bad < num_old_bad:
+            # increase step
+            step /= gamma
+        elif num_bad > num_old_bad:
+            # decrease step
+            step *= gamma
+        elif num_bad == num_old_bad:
+            # algorithm appears stuck
+            # increase step
+            step /= 0.8
+
+        # perturb step % of minimum mesh size
+        p[move] += step * h0 * perturb
+
+        # reproject boundary points
+        if sliver_opts["preserve"]:
+            p = _improve_level_set_newton(p, t, fd, deps, deps * 1000)
 
         count += 1
 
+        num_old_bad = len(ele_nums)
+
         end = time.time()
         if comm.rank == 0:
             print_msg2("     Elapsed wall-clock time %f : " % (end - start))
@@ -316,6 +339,9 @@ def generate_mesh(domain, edge_length, comm=None, **kwargs):  # noqa: C901
             Psuedo-timestep to control movement of points (default=0.30)
         * *geps_mult* (`float`) --
             The tolerance used to determine if a vertex is "in" the domain. (default==0.1*h0)
+        * *mesh_improvement* (`boolean`) --
+            Whether or not to run a density-preserving Laplacian smoothing to improve mesh quality
+            at the end of mesh generation (default=True)
 
 
     :return: points: vertex coordinates of mesh
@@ -336,6 +362,7 @@ def generate_mesh(domain, edge_length, comm=None, **kwargs):  # noqa: C901
         "delta_t": 0.30,
         "geps_mult": 0.1,
         "subdomains": None,
+        "mesh_improvement": True,
     }
     # check call was correct
     gen_opts.update(kwargs)
@@ -355,7 +382,7 @@ def generate_mesh(domain, edge_length, comm=None, **kwargs):  # noqa: C901
     # unpack domain
     fd, bbox0, corners = _unpack_domain(domain, gen_opts)
 
-    fh, bbox1, hmin = _unpack_sizing(edge_length)
+    fh, bbox1, hmin, lsf = _unpack_sizing(edge_length)
 
     # ensure consensus re hmin
     hmin = comm.bcast(hmin, 0)
@@ -415,7 +442,7 @@ def generate_mesh(domain, edge_length, comm=None, **kwargs):  # noqa: C901
         print_msg1("Constraining " + str(nfix) + " fixed points..")
 
     fh, p, extents = _initialize_points(
-        dim, geps, bbox, fh, fd, h0, gen_opts, pfix, comm
+        dim, geps, bbox, fh, fd, h0, gen_opts, pfix, comm, lsf
     )
 
     if gen_opts["max_iter"] < 0:
@@ -444,11 +471,11 @@ def generate_mesh(domain, edge_length, comm=None, **kwargs):  # noqa: C901
         start = time.time()
 
         # Remove non-unique points
-        p = np.array(list(set(tuple(p) for p in p)))
+        # p = np.array(list(set(tuple(p) for p in p)))
 
         # (Re)-triangulation by the Delaunay algorithm
         dt = DT()
-        dt.insert(p.flatten().tolist())
+        dt.insert(p.ravel().tolist())
 
         # Get the current topology of the triangulation
         p, t = _get_topology(dt)
@@ -547,12 +574,15 @@ def _check_bbox(bbox):
 def _unpack_sizing(edge_length):
     bbox = None
     hmin = None
+    localize_sizing_function = False
     if isinstance(edge_length, sizing.SizeFunction):
         bbox = edge_length.bbox
         fh = edge_length.eval
         hmin = edge_length.hmin
+        localize_sizing_function = True
     elif callable(edge_length):
         fh = edge_length
+        localize_sizing_function = False
     elif np.isscalar(edge_length):
 
         def func(x):
@@ -564,12 +594,13 @@ def _unpack_sizing(edge_length):
 
         fh = func
         hmin = edge_length
+        localize_sizing_function = True
     else:
         raise ValueError(
             "`edge_length` must either be a function, a `edge_length` object, or a scalar"
         )
     _check_bbox(bbox)
-    return fh, bbox, hmin
+    return fh, bbox, hmin, localize_sizing_function
 
 
 def _unpack_domain(domain, opts):
@@ -586,6 +617,7 @@ def _unpack_domain(domain, opts):
         geometry.Union,
         geometry.Intersection,
         geometry.Difference,
+        geometry.Repeat,
     ]
     if np.any([isinstance(domain, d) for d in domains]):
         bbox = domain.bbox
@@ -620,6 +652,9 @@ def _parse_kwargs(kwargs):
             "h0",
             "geps_mult",
             "subdomains",
+            "gamma",
+            "preserve",
+            "mesh_improvement",
         }:
             pass
         else:
@@ -641,8 +676,9 @@ def _termination(p, t, opts, comm, sliver=False, verbose=1):
             p, t, dim=2, min_qual=0.15, verbose=verbose
         )
         # only do Laplacian smoothing if no immersed domains
-        if opts["subdomains"] is None:
-            p, t = geometry.laplacian2(p, t, verbose=verbose)
+        if opts["subdomains"] is None and opts["mesh_improvement"]:
+            p, t = geometry.laplacian2_fixed_point(p, t)
+            # p, t = geometry.laplacian2(p, t, verbose=verbose)
     # perform linting if asked
     if comm.rank == 0 and opts["perform_checks"]:
         p, t = geometry.linter(p, t, dim=dim)
@@ -695,7 +731,7 @@ def _add_ghost_vertices(p, t, dt, extents, comm):
     exports = migration.enqueue(extents, p, t, comm.rank, comm.size, dim=dim)
     recv = migration.exchange(comm, comm.rank, comm.size, exports, dim=dim)
     recv_ix = len(recv)
-    dt.insert(recv.flatten().tolist())
+    dt.insert(recv.ravel().tolist())
     p, t = _get_topology(dt)
     p, t, inv = geometry.remove_external_entities(
         p,
@@ -780,13 +816,16 @@ def _user_defined_points(dim, fh, h0, bbox, points, comm, opts):
     return fh, p, extents
 
 
-def _generate_initial_points(h0, geps, dim, bbox, fh, fd, pfix, comm, opts):
+def _generate_initial_points(h0, geps, dim, bbox, fh, fd, pfix, comm, opts, lsf):
     """User did not specify initial points"""
-    if comm.size > 1:
+    if comm.size > 1 and lsf:
         # Localize mesh size function grid.
         fh = migration.localize_sizing_function(fh, h0, bbox, dim, opts["axis"], comm)
         # Create initial points in parallel in local box owned by rank
         p = mutils.make_init_points(bbox, comm.rank, comm.size, opts["axis"], h0, dim)
+    elif not lsf:
+        # Create initial points in parallel in local box owned by rank
+        p = mutils.make_init_points(bbox, comm.rank, comm.size, opts["axis"], h0, dim)
     else:
         # Create initial distribution in bounding box (equilateral triangles)
         p = mutils.create_staggered_grid(h0, dim, bbox)
@@ -809,12 +848,12 @@ def _generate_initial_points(h0, geps, dim, bbox, fh, fd, pfix, comm, opts):
     return fh, p, extents
 
 
-def _initialize_points(dim, geps, bbox, fh, fd, h0, opts, pfix, comm):
+def _initialize_points(dim, geps, bbox, fh, fd, h0, opts, pfix, comm, lsf):
     """Form initial point set to mesh with"""
     points = opts["points"]
     if points is None:
         fh, p, extents = _generate_initial_points(
-            h0, geps, dim, bbox, fh, fd, pfix, comm, opts
+            h0, geps, dim, bbox, fh, fd, pfix, comm, opts, lsf
         )
     else:
         fh, p, extents = _user_defined_points(dim, fh, h0, bbox, points, comm, opts)
