@@ -5,8 +5,27 @@ from .cpp.fast_geometry import (
     calc_dihedral_angles,
     calc_volume_grad,
     unique_edges,
+    drectangle_fast,
+    dblock_fast,
+    remove_external_entities2,
+    remove_external_entities3,
+)
+from .signed_distance_functions import (
+    Rectangle,
+    Cube,
+    Cylinder,
+    Disk,
+    Ball,
+    Torus,
+    Prism,
+    drectangle,
+    dblock,
+    corners,
+    Union,
+    Intersection,
+    Difference,
+    Repeat,
 )
-from .signed_distance_functions import Rectangle, Cube, Disk, drectangle, dblock
 from .utils import (
     calc_re_ratios,
     delete_boundary_entities,
@@ -21,6 +40,7 @@ from .utils import (
     get_facets,
     get_winded_boundary_edges,
     laplacian2,
+    laplacian2_fixed_point,
     linter,
     remove_external_entities,
     simp_qual,
@@ -28,7 +48,6 @@ from .utils import (
     unique_rows,
     vertex_to_entities,
     vertex_in_entity3,
-    corners,
 )
 
 __all__ = [
@@ -39,7 +58,13 @@ __all__ = [
     "calc_4x4determinant",
     "corners",
     "Rectangle",
+    "Ball",
     "Cube",
+    "Cylinder",
+    "drectangle_fast",
+    "dblock_fast",
+    "Torus",
+    "Prism",
     "Disk",
     "drectangle",
     "dblock",
@@ -47,6 +72,7 @@ __all__ = [
     "do_any_overlap",
     "linter",
     "laplacian2",
+    "laplacian2_fixed_point",
     "vertex_to_entities",
     "remove_external_entities",
     "unique_rows",
@@ -64,4 +90,10 @@ __all__ = [
     "get_winded_boundary_edges",
     "vertex_in_entity3",
     "unique_edges",
+    "Union",
+    "Intersection",
+    "Difference",
+    "Repeat",
+    "remove_external_entities3",
+    "remove_external_entities2",
 ]
