@@ -1,8 +1,12 @@
 import numpy as np
 import scipy.sparse as spsparse
 
+# from scipy.sparse.linalg import spsolve
+import pyamg
+
 from ..generation.cpp import c_cgal
-from . import signed_distance_functions as sdf
+
+# from . import signed_distance_functions as sdf
 from .cpp import fast_geometry as gutils
 
 # cpp implementation of 4x4 determinant calc
@@ -25,7 +29,7 @@ def calc_re_ratios(vertices, entities, dim=2):
         bars = np.concatenate(
             [entities[:, [0, 1]], entities[:, [1, 2]], entities[:, [2, 0]]]
         )
-        cc = c_cgal.circumballs2(vertices[entities.flatten()])
+        cc = c_cgal.circumballs2(vertices[entities.ravel()])
     elif dim == 3:
         bars = np.concatenate(
             [
@@ -68,33 +72,25 @@ def remove_external_entities(vertices, entities, extent, dim=2):
     :return: jx: mapping from old point indexing to new point indexing
     :rtype: numpy.ndarray[`int` x 1]
     """
-
     if dim == 2:
-        signed_distance = sdf.drectangle(
-            vertices[entities.flatten(), :],
-            x1=extent[0],
-            x2=extent[2],
-            y1=extent[1],
-            y2=extent[3],
+        signed_distance = gutils.remove_external_entities2(
+            vertices, entities, extent[0], extent[2], extent[1], extent[3]
         )
     elif dim == 3:
-        signed_distance = sdf.dblock(
-            vertices[entities.flatten(), :],
-            x1=extent[0],
-            x2=extent[3],
-            y1=extent[1],
-            y2=extent[4],
-            z1=extent[2],
-            z2=extent[5],
+        signed_distance = gutils.remove_external_entities3(
+            vertices,
+            entities,
+            extent[0],
+            extent[3],
+            extent[1],
+            extent[4],
+            extent[2],
+            extent[5],
         )
     isOut = np.reshape(signed_distance > 0.0, (-1, (dim + 1)))
-    entities_new = entities[(np.sum(isOut, axis=1) != (dim + 1))]
-    vertices_new = vertices
-    jx = np.arange(len(vertices_new))
-    # vertices_new, entities_new, jx = fix_mesh(
-    #    vertices, entities_new, dim=dim, fix_orientation=False
-    # )
-    return vertices_new, entities_new, jx
+    entities = entities[(np.sum(isOut, axis=1) != (dim + 1))]
+    jx = np.arange(len(vertices))
+    return vertices, entities, jx
 
 
 def vertex_to_entities(vertices, entities, dim=2):
@@ -494,6 +490,62 @@ def _sparse(Ix, J, S, shape=None, dtype=None):
     return spsparse.coo_matrix((S, (II, J)), shape, dtype)
 
 
+def laplacian2_fixed_point(vertices, entities):
+    """Solve the laplacian smoothing problem as a fixed point problem
+    solve this once, i.e., (I - A) x = rhs vs.
+    repeating this x_{n+1} = A x_n
+
+    :param vertices: vertex coordinates of mesh
+    :type vertices: numpy.ndarray[`float` x dim]
+    :param entities: the mesh connectivity
+    :type entities: numpy.ndarray[`int` x (dim+1)]
+
+    :return vertices: updated vertices of mesh
+    :rtype: numpy.ndarray[`float` x dim]
+    :return: entities: updated mesh connectivity
+    :rtype: numpy.ndarray[`int` x (dim+1)]
+    """
+    if vertices.ndim != 2:
+        raise NotImplementedError("Laplacian smoothing is only works in 2D for now")
+
+    n = len(vertices)
+
+    nds = entities.T
+    local_idx = np.array([[1, 2], [2, 0], [0, 1]]).T
+    idx = nds[local_idx]
+    row_idx = np.array([idx[0], idx[1], idx[0], idx[1]]).flat
+    col_idx = np.array([idx[0], idx[1], idx[1], idx[0]]).flat
+
+    a = np.ones(idx.shape[1:], dtype=int)
+    val = np.array([+a, +a, -a, -a]).flat
+
+    # Create CSR matrix for efficiency
+    matrix = spsparse.coo_matrix((val, (row_idx, col_idx)), shape=(n, n))
+    matrix = matrix.tocsr()
+
+    bnd = get_boundary_vertices(entities)
+
+    # Apply Dirichlet conditions.
+    # Set all Dirichlet rows to 0.
+    for b in bnd:
+        matrix.data[matrix.indptr[b] : matrix.indptr[b + 1]] = 0.0
+    # Set the diagonal and RHS.
+    d = matrix.diagonal()
+    d[bnd] = 1.0
+    matrix.setdiag(d)
+
+    rhs = np.zeros((n, 2))
+    rhs[bnd] = vertices[bnd]
+
+    # vertices_new = spsolve(matrix, rhs)
+
+    # use AMG
+    ml = pyamg.ruge_stuben_solver(matrix)
+    vertices_new = np.column_stack([ml.solve(rhs[:, 0]), ml.solve(rhs[:, 1])])
+
+    return vertices_new, entities
+
+
 def laplacian2(vertices, entities, max_iter=20, tol=0.01, verbose=1):
     """Move vertices to the average position of their connected neighbors
     with the goal to hopefully improve geometric entity quality.
