@@ -1,14 +1,16 @@
 # run EAGE benchmark and plot a figure
+import matplotlib
 import matplotlib.pyplot as plt
 import numpy
 from benchmark_EAGE import run_gmsh, run_SeismicMesh, _build_sizing
 
+matplotlib.use("TkAgg")
 
-colors1 = ["ko-", "ro-", "bo-"]
-colors2 = ["ko--", "ro--", "bo--"]
-labels = ["gmsh", "SeismicMesh", "cgal"]
+colors1 = ["C7o-", "C3o-", "C0o-"]
+colors2 = ["C7o--", "C3o--", "C0o--"]
+labels = ["Gmsh", "SeismicMesh", "CGAL"]
 
-plt.rcParams.update({"font.size": 18})
+# plt.rcParams.update({"font.size": 18})
 
 entries = []
 # minimal mesh size
@@ -44,18 +46,27 @@ for i, func in enumerate([run_gmsh, run_SeismicMesh]):
     entries.append(h)
 
 plt.subplot(1, 2, 1)
-plt.title("Number of cells vs. mesh generation time")
+plt.title("# of cells vs. mesh generation time [s]")
 plt.ticklabel_format(style="sci", axis="x", scilimits=(0, 0))
 plt.legend()
-plt.xlabel("Number of cells")
-plt.ylabel("Elapsed time (s)")
-plt.grid()
+plt.xlabel("# of cells")
+plt.ylim(ymax=60, ymin=0)
+
+axes = plt.gca()
+axes.yaxis.grid()
+axes.set_frame_on(False)
+
 
 plt.subplot(1, 2, 2)
-plt.title("Number of cells. vs cell quality")
+plt.title("# of cells. vs cell quality")
 plt.ticklabel_format(style="sci", axis="x", scilimits=(0, 0))
-plt.xlabel("Number of cells")
-plt.ylabel("Cell quality")
-plt.grid()
+plt.xlabel("# of cells")
+plt.ylim(ymax=1, ymin=0)
 
-plt.show()
+axes = plt.gca()
+axes.yaxis.grid()
+axes.set_frame_on(False)
+
+
+plt.tight_layout(pad=3.0)
+plt.savefig("out4.svg", transparent=True, bbox_inches="tight", pad_inches=0)
